Ext.define('DSS.editor.PowerStateEditorPart', {
	extend : 'DSS.component.Base',
	mixins : {
		observable : 'Ext.util.Observable'
	},
	initStores: function() {
        if (Ext.data.StoreManager.lookup('powerState_dsm') == null) {
            Ext.create('Ext.data.Store', {
                storeId : 'powerState_dsm',
                fields : [ 'dsid', 'name'],
                proxy : {
                    type : 'memory',
                    reader : {
                        type : 'json',
                        root : 'items'
                    }
                }
            });
            
            for (var iIndex=0;iIndex<dss.staticDataModel.MeterCache.length;iIndex++) {
    			for (var jIndex=0;jIndex<dss.staticDataModel.MeterCache[iIndex].capabilities.length;jIndex++)
    				if (dss.staticDataModel.MeterCache[iIndex].capabilities[jIndex].metering)
    					Ext.data.StoreManager.lookup('powerState_dsm').add({dsid:dss.staticDataModel.MeterCache[iIndex].dSUID,name:dss.staticDataModel.getMeterName(dss.staticDataModel.MeterCache[iIndex].dSUID)});
            }
        }

	},
	getHeight : function() {
		return 0;
	},
	constructor : function(config) {	
		this.initStores();
		var me=this;
		this.dsidControl=Ext.create("DSS.component.Textedit",{			            
            fieldLabel : _('dsuid(plain)'),
    		labelWidth:80,
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            width : 360,
            inputWidth : 250,
		});

		this.upperValue=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            fieldLabel:_('Powervalue for setting'),
            width : 320,
            labelWidth:260,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 3600,
            margin: "0 0 0 5",
            autoStripChars:true,
            decimalPrecision:0
        });
		this.lowerValue=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            fieldLabel:_('Powervalue for resetting'),
            width : 320,
            labelWidth:260,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 3600,
            margin: "10 0 0 5",
            autoStripChars:true,
            decimalPrecision:0
        });
		
		this.dsmGrid = Ext.create('Ext.grid.Panel', {
            hideHeaders:true,
            height : 150,
            width : 350,
            autoScroll : true,
            viewConfig : {
                stripeRows : true
            },
            selModel : {
                mode : 'MULTI'
            },
            store : Ext.data.StoreManager.lookup('powerState_dsm'),
            columns : [ {
                sortable : false,
                hideable : false,
                dataIndex : 'name',
                flex : 1
            } ]
        });
		
		this.dsmPanel=Ext.create('Ext.panel.Panel', {
			border:0,
			items : [this.dsmGrid],
			height: 150,
			width:365,
			layout: {
				type:'vbox'
			}
		});
		
		this.valuePanel=Ext.create('Ext.panel.Panel', {
			border:0,
			items : [this.upperValue,{xtype:'displayfield',value:('W'), margin: "0 0 0 5", width:10},
			         this.lowerValue,{xtype:'displayfield',value:('W'), margin: "10 0 0 5", width:10}],
			height: 150,
			width:360,
			layout: {
				type:'table',
				columns:2
			}
		});
		
		this.setRetriggeringMinute=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            fieldLabel:_('Set state after a period of'),
            width : 320,
            labelWidth:270,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 59,
            margin: "8 0 0 5",
            autoStripChars:true,
            decimalPrecision:0
        });
		this.setRetriggeringSeconds=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            width : 44,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 59,
            margin: "8 0 0 0",
            autoStripChars:true,
            decimalPrecision:0
        });
		
		this.resetRetriggeringMinute=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            fieldLabel:_('Reset state after a period of'),
            width : 320,
            labelWidth:270,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 59,
            margin: "8 0 0 5",
            autoStripChars:true,
            decimalPrecision:0
        });
		this.resetRetriggeringSeconds=Ext.create('Ext.form.field.Number', {
            allowBlank : false,
            blankText : _('This field is required'),
            invalidText : _('The value in this field is invalid'),
            width : 44,
            step : 1,
            value : 0,
            minValue : 0,
            maxValue : 59,
            margin: "8 0 0 0",
            autoStripChars:true,
            decimalPrecision:0
        });
		
		this.retriggerPanel=Ext.create('Ext.panel.Panel', {
			border:0,
			items : [this.deviceUnready, 
			         this.setRetriggeringMinute,	         
			         this.setRetriggeringSeconds,
			         {xtype:'displayfield',value:_('mm:ss'), margin: "8 0 0 5", width:50},
			         {xtype:'displayfield',value:_('if no other event was received anymore'), margin: "8 0 0 5", width:370}, 
			         this.resetRetriggeringMinute,	         
			         this.resetRetriggeringSeconds,
			         {xtype:'displayfield',value:_('mm:ss'), margin: "8 0 0 5", width:50},
			         {xtype:'displayfield',value:_('if no other event was received anymore'), margin: "8 0 0 5", width:370}
			         ],
			height: 80,
			width:738,
			layout: {
				type:'table',
    			columns:4
			},
			colspan:2
		});
		
		Ext.apply(this, {
        	border: 0,
        	margin:"5 0 0 0",
  			items : [
  				{xtype:'displayfield',value:_('Please select a dSM and the powerlevels for setting and resetting.'), margin: "0 0 10 0",colspan:2}
  			,this.dsmPanel,this.valuePanel,this.retriggerPanel ],
  			height: 250,
  			width:740,
  			layout: {
  				type: 'table',
  				columns:2
  			}
  		});
    	
 		this.callParent(arguments);
	},
	reset : function() {
        this.dsmGrid.getSelectionModel().deselectAll();
        this.dsmGrid.getEl().child('.x-panel-body').applyStyles('border: solid 1px #99e8bc;');
		this.lowerValue.clearInvalid(null);
		this.upperValue.clearInvalid(null);
		this.lowerValue.setRawValue(null);
		this.upperValue.setRawValue(null);
		this.setRetriggeringMinute.setValue(0);
		this.setRetriggeringSeconds.setValue(0);
		this.resetRetriggeringMinute.setValue(0);
		this.resetRetriggeringSeconds.setValue(0);
	},
	isValid : function() {
		if (this.isHidden())
			return true;
		if (this.dsmGrid.getSelectionModel().getSelection().length!=1) {
			this.dsmGrid.getEl().child('.x-panel-body').applyStyles('border: solid 1px red')
			return false;
		} else {
			this.dsmGrid.getEl().child('.x-panel-body').applyStyles('border: solid 1px #99e8bc;');
		} 
		var fLower=this.lowerValue.validate();
		var fUpper=this.upperValue.validate();
		return (fLower && fUpper);
	},
	setValue : function(oValue) {
		if (this.isHidden())
			return;
		this.reset();
		var oRecord=this.dsmGrid.store.find('dsid',oValue.dsid,0,false,false,true);
        if (oRecord != -1) {
        	this.dsmGrid.getSelectionModel().select(oRecord,true);
        }
		this.lowerValue.setValue(oValue.lowerValue);
		this.upperValue.setValue(oValue.upperValue);
        this.setRetriggeringSeconds.setValue(oValue.setRetriggeringLag%60);
        this.resetRetriggeringSeconds.setValue(oValue.resetRetriggeringLag%60);
        this.setRetriggeringMinute.setValue(Math.floor(oValue.setRetriggeringLag/60));
        this.resetRetriggeringMinute.setValue(Math.floor(oValue.resetRetriggeringLag/60));
	},
	getValue : function() {
		if (this.isHidden())
			return null;
		return {
			dsid:this.dsmGrid.getSelectionModel().getSelection()[0].get('dsid'),
			lowerValue:this.lowerValue.getValue(),
			upperValue:this.upperValue.getValue(),
			setRetriggeringLag:this.setRetriggeringSeconds.getValue() + (this.setRetriggeringMinute.getValue()*60),
			resetRetriggeringLag:this.resetRetriggeringSeconds.getValue() + (this.resetRetriggeringMinute.getValue()*60)
		};
	}
});